package kfmes.jateon.swing;

import java.awt.Component;
import java.awt.Color;
import java.util.Enumeration;
import javax.swing.*;
import javax.swing.tree.*;

import kfmes.natelib.entity.*;


/**
 * 
 * @author kfmes(KIM Ga-Hyeon, jateon@kfmes.com, http://jateon.kfmes.com )
 * @version $Id: BuddyRenderer.java,v 1.18 2007/11/19 10:13:35 kfmes Exp $
 */
public class BuddyRenderer extends DefaultTreeCellRenderer
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public static final int VIEW_NAME = 0;
	
	public static final int VIEW_NICK= 1;
	public static final int VIEW_ID = 2;
	
	public static final int VIEW_NAME_ID = 3;
	public static final int VIEW_NAME_NICK = 4;

	private NateFriend friend = null;
	private Color selectedBack = new Color( 0x666699 );
	private Color selectedFore = Color.white;

	/**
	 * Determine buddy's friendly name has shown. or login name.
	 */
	private int viewMode = VIEW_NICK;

	private ImageIcon imageOnline, imageOffline, imageBusy, imageAway;

	public BuddyRenderer()
	{
		setOpaque( true );
		//setFont( FONT_10 );

		imageOnline = UIToolkit.getImage("icon/on.gif");
		imageOffline = UIToolkit.getImage("icon/off.gif");
		imageBusy = UIToolkit.getImage("icon/busy.gif");
		imageAway = UIToolkit.getImage("icon/away.gif");
		/*
		imageOnline = new ImageIcon(getIconResource("on.gif"));
		imageOffline = new ImageIcon(getIconResource("off.gif"));
		imageBusy = new ImageIcon(getIconResource("busy.gif"));
		imageAway = new ImageIcon(getIconResource("away.gif"));
		*/
	}


	public void setBuddyView( int mode )
	{
		this.viewMode = mode;
	}

	public int getBuddyView()
	{
		return this.viewMode;
	}
/*
	private URL getIconResource( String name )
	{
		//return this.getClass().getResource( "resources/icon/" + name );
		try {
			return new URL(UIToolkit.resPath + "icon/" + name );
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
		return null;
	}
*/
	protected String getGroupText( boolean offView, DefaultMutableTreeNode groupNode, String str )
	{
		
		int total = groupNode.getChildCount();
		int online = 0;
		for(Enumeration e=groupNode.children(); e.hasMoreElements(); )
		{
			NateFriend friend = (NateFriend)
				(((DefaultMutableTreeNode)e.nextElement()).getUserObject());
			if( !friend.getStatus().equals(UserStatus.Offline) )
				online++;
		}
		
		return  (offView)? str + " (" + online + "/" + total + ")" : str + "(" + online + ")";
	}
	
	public Component getTreeCellRendererComponent( JTree tree, Object value, boolean sel, 
		boolean expanded, boolean leaf, int row, boolean hasFocus )
	{
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
		Object obj = node.getUserObject();
		if( obj instanceof NateFriend )
		{	
			this.friend = (NateFriend)obj;
			String status = friend.getStatus();
			if( status==null || status.equals(UserStatus.Online) )
			{
				status = "";
				setIcon( imageOnline );
			}
			else
			{
				setIcon( imageOnline );

				if( status.equals(UserStatus.Away) ||
					status.equals(UserStatus.Meeting) )
				{
					setIcon( imageAway );
				}
				else
				if( status.equals(UserStatus.Busy) ||
					status.equals(UserStatus.Phone))
				{
					setIcon( imageBusy );
				}
				else
				if( status.equals(UserStatus.Offline) ||
					status.equals(UserStatus.Invisible) )
				{
					setIcon( imageOffline );
				}

				status = " (" + friend.getFormattedStatus() + ")";
			}

			switch( viewMode )
			{
			case VIEW_NAME:
				setText( friend.getRealName().concat(status));
				break;
			case VIEW_NICK:
				setText( friend.getNickName().replaceAll("%20", " ").replaceAll("%25", "%").concat(status) );
				break;
			case VIEW_ID:
				setText(friend.getID().concat(status));
				break;
			case VIEW_NAME_ID:
				setText( friend.getRealName() + "(" + friend.getID() + ")" + status);
				break;
			case VIEW_NAME_NICK:
				setText( friend.getRealName() + "(" + friend.getNickName().replaceAll("%20", " ").replaceAll("%25", "%") + ")" + status);
				break;
				/*
			case VIEW_LOGIN_NAME:
				setText( friend.getNameID() .getLoginName().concat(status) );
				break;*/
			}
		}
		else
		if( obj instanceof NateGroup )
		{
			NateGroup group = (NateGroup)obj;
			setIcon( null );
			BuddyTree t = (BuddyTree)tree;
			
			setText( getGroupText(t.getOffview(), node, group.getName() ) );
		}

		setBackground( sel ? selectedBack : Color.white );
		setForeground( sel ? selectedFore : Color.black );

		return this;
	}
}