/**
 * @file ${file_name} The Jateon Messenger
 *
 * ${package_name}
 *
 * ${package_name} is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package kfmes.jateon.swt;

import java.util.StringTokenizer;


import kfmes.jateon.swt.dialog.AddFriendDialog;
import kfmes.jateon.swt.dialog.ChatWindow;
import kfmes.jateon.swt.dialog.InputDialog;
import kfmes.jateon.swt.dialog.JateonSMS;
import kfmes.jateon.swt.dialog.LoginDialog;
import kfmes.jateon.swt.dialog.OptionDialog;
import kfmes.jateon.swt.dialog.SendMessage;
import kfmes.jateon.swt.dialog.WebBrowser;
import kfmes.natelib.NateonMessenger;
import kfmes.natelib.SwitchBoardSession;
import kfmes.natelib.entity.NateFriend;
import kfmes.natelib.entity.NateGroup;
import kfmes.natelib.entity.UserStatus;
import kfmes.natelib.util.HtmlReader;
import kfmes.natelib.util.MsgUtil;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;

/**
 * UI 에서 발생되는 액션들을 모아놓은 클래스
 * @author kfmes(KIM Ga-Hyeon, jateon@kfmes.com, http://jateon.kfmes.com )
 * @version $Id: ActionGroup.java,v 1.44 2007/11/19 10:13:31 kfmes Exp $
 */
final public class ActionGroup {
	private MainFrame main = null;
	private NateonMessenger nate = null;
	private BuddyTree buddies = null;
	private ConfUtil conf = null;
	//private MessageBox msgBox = null;
	

	public ActionGroup( MainFrame main, BuddyTree buddies )
	{
		this.main = main;
		this.buddies = buddies;
		this.nate = main.getMessenger();
		this.conf = main.getConf();
		
		/*
		msgBox = new MessageBox(main.shell, SWT.ICON_WARNING);
		msgBox.setText("정보");
		*/


	}
	public void setBuddyTree(BuddyTree buddies){
		this.buddies = buddies;
	}

	/**
	 * 알파 & 베타 테스트 기간동안 서버에 기록을 남기기 위한 코드
	 * http://155.230.165.20/~kfmes/jateonLog/";
	 * access.php?cver=xxx
	 * @param loginName
	 */
	@SuppressWarnings("static-access")
	protected void actionProcessLogin(String loginName){
		{
			String url = kfmes.jateon.swt.Main.authServer;
			//url += "access.php?id=" + loginName + "&cver=" + kfmes.jateon.swt.Main.ver; 
			url += "access.php?&cver=" + kfmes.jateon.swt.Main.ver;
			
			HtmlReader reader = new HtmlReader(url,"UTF8");
			String result;
			result = reader.getHTML().trim();
		
			String resultar[] = null;
			
			resultar = result.split("\n");
			result = result.replaceAll("\n", "\r\n");
			
			
			if(!resultar[0].equals("OK")){
				if(resultar[0].equals("Warning"))
					main.showMessageBox("경고", result.substring(result.indexOf("\n")+1), SWT.ICON_WARNING);
				else if(resultar[0].equals("Block"))
					main.showMessageBox("이용제한", result, SWT.ICON_ERROR);
				else
					main.showMessageBox("정보", "최신버전 정보를 가져오지 못했습니다", SWT.ICON_INFORMATION);
			}
			if(resultar[0].equals("OK") && resultar.length>1)
				main.showMessageBox("정보",  result.substring(result.indexOf("\n")+1), SWT.ICON_INFORMATION);
				
			
			
		
		}
		
		/**
		 * access log code 끝
		 */
				
	}
	
	@SuppressWarnings("static-access")
	public boolean actionLogin()
	{
		new LoginDialog(main);
		
		if(LoginDialog.isReady){
			String loginName = conf.getString("loginName");
			String passwd = conf.getString("passwd");
			nate.is_encodedpasswd = true;
			main.menuBar.setOnline(true);
			main.shell.setVisible(true);
			
			actionProcessLogin(loginName);
			nate.login(loginName,passwd);	
			//main.menuBar.setOnline(true);
			return true;
		}
		main.menuBar.setOnline(false);
		return false;
	}

	public void actionLogout()
	{
		try{
			

			conf.set("FrameLocation", main.getFrameInfo());
			conf.set("offview", (buddies.getOffView() ? "yes":"no"));
			conf.set("viewmode", buddies.getBuddyView() + "" );
			//conf.set("LookAndFeel", main.currentLookAndFeel);
			conf.set("DefaultAction", main.getDblClickDefaultAction() + "");
			conf.set("ChatWindowSize", main.getChatWinSize().x + " " + main.getChatWinSize().y  );
			conf.writeData();
			
			buddies.tree.removeAll();
			
			if(!nate.getNS().isConnected())
				return;
			//nate.setMyStatus(UserStatus.Offline);
			main.setUserStatusNull();
			nate.logout();

			main.menuBar.setOnline(false);
		}
		catch (NullPointerException e) {

		}		
		catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	public void actionQuit(){
		
		actionLogout();
		if(main.tray!=null)
			main.tray.disopse();
		System.exit(0);
	}
	@SuppressWarnings("static-access")
	public void actionClearLoginInfo(){
		if( main.showMessageBox("경고", "저장되어있던 로그인 정보와" + 
				"\n 설정 데이터가 지워집니다.\n계속하시겠습니까?", SWT.YES|SWT.NO|SWT.ICON_WARNING)
				==SWT.YES)		{

			ConfUtil conf = main.getConf();
			conf.removeAll();
			conf.writeData();
			
		}

		
	}
	
	
	/*
istener( action.get("actionSendIMsg") );
			mReqChat.addActionListener( action.get("actionReqChat") );
	 */

	
	public void  actionSendIMsg(NateFriend fr){
		if(nate.isLoggedIn() ){
			if(fr==null)
				fr = buddies.getSelectedFriend();
			if(fr!=null)
				new SendMessage(main, nate , fr);			
			//send.setRecvUser(getSelectedUser());
		}
	
	}
	
	public void actionReqChat(NateFriend fr){
		
		if(nate.isLoggedIn()){
			
			if(fr==null)
				fr = buddies.getSelectedFriend();
			if(fr!=null){
				
				
				SwitchBoardSession ss =nate.findSwitchboardSessionAt(fr.getNateID());
				ChatWindow cw = buddies.getChatWindow(ss);
				if(cw==null || cw.getSS()==null || !cw.getSS().isConnected()){
					cw = new ChatWindow( nate,buddies,ss, fr);
				//	cw.setIconImage(UIToolkit.getFrameIcon(UIToolkit.iconChat));
					
					//buddies.getChatWinList().add(cw);
				}
				else {
				
				cw.flashing();
				}
			}
		}	
	}
	public void actionSendSMS(){
		if(nate.isLoggedIn()){
			//JateonSMS sms = 
			new JateonSMS(nate,main);		
			
		}
		
	}
	public void actionVisitMiniHP(String cyworldNum){
		
		//new WebBrowser(main, "http://cyworld.com/pims/main/pims_main_nateon.asp?tid=" + cyworldNum, 
//				WebBrowser.MODE_CYMINIHP);
		String url = "http://br.nate.com/index.php?code=D002&t=" + nate.getTicket() + "&param=" + cyworldNum;
		if(Main.osname.startsWith("mac") || Main.osname.startsWith("win"))
			new WebBrowser(main, url ,//"http://cyworld.com/pims/main/pims_main_nateon.asp?tid=" + cyworldNum, 
					WebBrowser.MODE_CYMINIHP);
		else
			new WebBrowser(true, url // "http://cyworld.com/pims/main/pims_main_nateon.asp?tid=" + cyworldNum);
					);
		
		//	new WebBrowser("http://cyworld.com/pims/main/pims_main_nateon.asp?tid=" + m.getData());
	}
	
	/* XXX:???
	public void actionProcessNotify(){
		//buddies.lastLoginBuddy
		NateFriend fr = buddies.lastLoginBuddy;
		if(main.getConfutil().getBoolean("DefaultAction")){
			SwitchBoardSession ss =nate.findSwitchboardSessionAt(fr.getNateID());
			ChatWindow cw = buddies.getChatWindow(ss, fr);
			if(cw==null || !cw.getSS().isConnected()){

				cw = new ChatWindow( nate,buddies,ss, fr);
			//	cw.setIconImage(UIToolkit.getFrameIcon(UIToolkit.iconChat));
				buddies.getChatWinList().add(cw);
			}
			else {
				cw.flashing();
			}
		}
		else{ //msg
			SendMessageGUI send = new SendMessageGUI(nate );			
			send.setRecvUser(fr);
			
		}
		
		
	}
	*/
	@SuppressWarnings("static-access")
	public void actionBuddyDbg() {
		//if(nate.isLoggedIn() ){
			NateFriend fr = buddies.getSelectedFriend();
			main.showMessageBox("Buddy info", fr.toString(), SWT.ICON_INFORMATION);		

			//send.setRecvUser(getSelectedUser());
		//}
	}
	
	@SuppressWarnings("static-access")
	public void actionAbout(){
		String txt = "JaTeOn " + Main.version + "\r\nip : " + nate.filemgr.localhost;
		
		main.showMessageBox("JaTeOn 정보", txt, SWT.ICON_INFORMATION);
		
	}
/*
	public final static String Online = "O";
	public final static String Away = "A";
	public final static String Busy = "B"; 
	public final static String Phone = "P";
	public final static String Meeting = "M";
	public final static String Invisible = "X";
	public final static String Offline = "F";
	Online", "Away", "Busy", "Phone", "Meeting", "Invisible" };
 */	
	
	@SuppressWarnings("static-access")
	public void actionChangeNick(){
		String newName = nate.getOwner().getNickName().replaceAll("%20", " ").replaceAll("%25", "%");

		try{
			InputDialog id = new InputDialog(main.shell , "닉네임 변경","변경할 닉네임을 입력하세요");
			id.setInput(newName);
			newName = id.open();
			
			//newName =new INPUT 
				//JOptionPane.showInputDialog(main, "변경할 닉네임을 입력하세요",  newName);
		
			//newName = jp.showInputDialog(main, "변경할 닉네임을 입력하세요", "닉네임변경", JOptionPane.QUESTION_MESSAGE);
			
			if(!nate.getOwner().getNickName().replaceAll("%20", " ").replaceAll("%25", "%").equals(newName))
				nate.setMyNickName(newName);
		}
		catch (Exception e) {
			// TODO: handle exception
		}
	}
	
	/**
	 * 라이센스 다이얼로그
	 *
	 */
	public void actionAboutLicense(){
		String txt =// "\tJaTeOn 라이센스\n\n" +
				"jateon 은 공식 포럼에서 구하실 수 있습니다.\n\n" +
				"" +
				"본 프로그램을 사용하여 발생하는 불이익 등에 대해서는\n" +
				" 제작자는 책임을 지지 않음을 알려드립니다.\n\n" +
				"" +
				"영리목적으로 사용 하시면 안됩니다\n" +
				"  (광고나 스펨 기타 등등)\n\n" +
				"" +
				"별도의 공지 없이 jateon 이용이 중지될수도 있습니다.";
		MainFrame.showMessageBox("JaTeOn 라이센스", txt, SWT.ICON_INFORMATION);
	}
	
	/**
	 * 사용자 상태 전환
	 */
	public void actionSetStatusOnline(){
		nate.setMyStatus(UserStatus.Online);
	}
	public void actionSetStatusAway(){
		nate.setMyStatus(UserStatus.Away);
	}
	public void actionSetStatusBusy(){
		nate.setMyStatus(UserStatus.Busy);
	}
	public void actionSetStatusPhone(){
		nate.setMyStatus(UserStatus.Phone);
	}
	public void actionSetStatusMeeting(){
		nate.setMyStatus(UserStatus.Meeting);
	}
	public void actionSetStatusInvisible(){
		nate.setMyStatus(UserStatus.Invisible);
	}
	public void actionSetStatusOffline(){
		nate.setMyStatus(UserStatus.Offline);
	}
	
	/**
	 * 오프라인 사용자 보기, 안보기 toggle
	 */
	public void actionToggleOffView()
	{
		buddies.toggleOfflineView();
	}

	/**
	 * 친구목록을 업데이트 해준다
	 */
	public void actionRefreshList(){
		if(nate.isLoggedIn())
			buddies.syncAllUsers();
	}
	
	/**
	 * 가비지 컬렉터 실행
	 */
	public void actionSystemGC(){
		System.gc();
	}
	
	/**
	 *  보기 모드 전환
	 */


	protected void actionSetViewMode(int mode){
		main.getBuddyTree().setBuddyView(mode);
		this.actionRefreshList();

	}


	
	public void actionAddFriend(){
		
	if(nate.isLoggedIn())
		{
			//System.out.println("-_-");
			//main.showMessageBox("미구현", "아직 구현 되지 않았습니다", SWT.ICON_WARNING);
			//FIXME : 
			AddFriendDialog buddyAdd = new AddFriendDialog(nate , main);
			/*
			ArrayList<NateGroup> l = nate.getBuddyGroup().getList();
			for(int i =0;i<l.size();i++)
				MsgUtil.println(l.get(i).getName() + "/" + l.get(i).getNumber());
				*/
			//buddyAdd.setVisible(true);
		}
	}
	public void actionWhoAddedMe(final NateFriend friend) {
		
		Display.getDefault().asyncExec(new Runnable(){
			public void run(){
				int ret  = MainFrame.showMessageBox("친구 등록 요청", 
						friend.getNameID() + "님이 당신을 친구로 등록했습니다." + (friend.getTag()!=null ? friend.getTag() : "") , 
						SWT.YES|SWT.NO|SWT.CANCEL | SWT.ICON_QUESTION);
				
			
				if (ret == SWT.YES) {
				 friend.setFABR(true, "FA");
				 //MsgUtil.println("ADD");
				 nate.reqAdsb(true, friend);	        	 
				
				} else if (ret == SWT.NO) {
				 friend.setFABR(true, "B");
				// MsgUtil.println("DENY");
				   	 nate.reqAdsb(false, friend);
				    } else {
				
				    
				    }
			}
		});
		
    }

	public void actionSetDefaultMsg(){
		//boolean flag = true;
		main.setDblClickDefaultAction(MainFrame.DefaultActionMsg);
	}
	public void actionSetDefaultChat(){
		//boolean flag = false;
		main.setDblClickDefaultAction(MainFrame.DefaultActionChat);
		
	}
	
	public void actionDetailSetting(){
		new OptionDialog(main,nate);
	}
	public void actionDebugInfo(){
		String msg = "TICKET=" + nate.getTicket() + "\nUserID=" + nate.getOwner().getNateID();
		MsgUtil.println(msg);
		conf.debugPrint();
		//MsgUtil.println(main.shell.getSize() + "/" +main.shell.getLocation());
		//MsgUtil.println(main.getFrameInfo());
	}
	@SuppressWarnings("static-access")
	public void actionVersionCheck(){
		String url = Main.authServer + "version_" 
		+ ((Main.releaseType.startsWith("Alpha")) ? "alpha" : "beta") 
		+".txt";
		
		HtmlReader reader = new HtmlReader(url,"UTF8");
		
		StringTokenizer token = new StringTokenizer(reader.getHTML(), "\n");
		String line = null;
		
		String item = null;
		String value =null;
		int idx = -1;
		
		String nversoin = null;
		String ndate = null;
		String ncomment = null;
		
		while(token.hasMoreElements()){
			item =null;
			value =null;
			line = token.nextToken();
			if(line.trim().startsWith("#"))
				continue;
			idx = line.indexOf('=');
			if(idx<0) 
				continue;
			item = line.substring(0,idx).trim();
			value = line.substring(idx+1).trim();

			if(item.equals("version"))
				nversoin = value;
			else if(item.equals("date"))
				ndate = value;
			else if(item.equals("comment"))
				ncomment = value;
		}
		String vinfo = "사용중인 버전 : " + Main.ver +
				"\n최신 버전 : " + nversoin + "\n릴리즈 날짜 : " + ndate + 	"\n내용 : " + ncomment;
		main.showMessageBox("버전 정보", vinfo , SWT.ICON_INFORMATION);
	}
	public void actionToggleFileMon() {
		main.fileTransferDlg.toogleVisible();
	}
	public void toggleBlock(NateFriend friend) {
		boolean block = friend.getFABR().charAt(2) =='1' ? true : false;
		nate.getNS().sendBlock(friend, !block);		
		
	}
	public void actionMoveGroup(NateFriend friend, String newg) {
		nate.moveGroup(friend, newg);
		
	}
	@SuppressWarnings("static-access")
	public void actionAddGroup() {
		try{
			String name = "새 그룹";
			InputDialog id = new InputDialog(main.shell , "그룹 추가","추가할 그룹이름을 입력하세요");
			id.setInput(name);
			name = id.open();
			nate.getNS().addGroup(name);
		}
		catch (Exception e) {
			// TODO: handle exception
		}
		
	}
	@SuppressWarnings("static-access")
	public void actionRenameGroup(NateGroup group) {
		try{
			String oldname = group.getName();
			String name = oldname;
			InputDialog id = new InputDialog(main.shell , "그룹 이름 변경","변경할 그룹이름을 입력하세요");
			id.setInput(name);
			name = id.open();
			if(name.equals(oldname))
				return;
			nate.getNS().renameGroup(group, name);
		}
		catch (Exception e) {
			// TODO: handle exception
		}
		
	}
	public void actionDeleteGroup(NateGroup group) {
		if(group.getList().size()>0)
		{
			MainFrame.showMessageBox("그룹 지우기 실패", "그룹 내에 친구가 있어 삭제할 수 없습니다.", SWT.ICON_INFORMATION);
			return;			
		}
		nate.getNS().removeGroup(group);
		
	}
	@SuppressWarnings("static-access")
	public void removeBuddy(NateFriend friend) {
		if( main.showMessageBox("친구 지우기 확인", "정말로 지우시겠습니까?", SWT.YES|SWT.NO|SWT.ICON_INFORMATION)
				==SWT.YES)		{
			nate.getNS().removeBuddy(friend);			
		}
		
	}
	

        


}