package kfmes.jateon.swt;

import java.io.*;
import java.util.Hashtable;
import javax.sound.sampled.*;

import kfmes.natelib.util.MsgUtil;

/**
 * 길에서 걸어다니면서 음악을 들을 수 있다는
 * 그 유명한 워크맨.
 * Wav 포맷 외에는 테스트해본적 없으니 주의.
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr /  Modified by kfmes, kfmes@kfmes.com
 * @version $Id: MusicBox.java, 2007/02/15 
 */
final public class MusicBox
{
	
	public static final String SOUND_STARTING = "starting";
	public static final String SOUND_LOGIN = "login";
	public static final String SOUND_LOGOUT = "logout";
	public static final String SOUND_MESSAGE = "message";
	public static final String SOUND_CHAT = "chat";
	public static final String SOUND_FILE = "file";
	

	private static LineListener clipListener = new ClipListener();
	private static Hashtable<String,byte[]> waveMap = new Hashtable<String,byte[]>();
	private static boolean useSound = true;
	
	private static long musicBoxLastPlayedTime = -1;
//	private static Hashtable acceptMap = new Hashtable();

	/**
	 * 설정을 읽어 각 사운드 데이터를 메모리로 올린다.
	 */
	public static void init()
	{
//		Class cl = MusicBox.class;

		try
		{	
			String respath =Main.getJateonPath() + "resources/sounds/";
			String urlSndStarting	= 	respath + "login.wav";
			String urlSndLogin 		= 	respath + "login.wav";
			String urlSndLogout	= 		respath + "logout.wav";
			String urlSndMessage= 	respath + "message.wav";
			String urlSndChat 		=	respath + "chat.wav"; 
			String urlSndFILE 		= 		respath + "file.wav";
			/*
			URL urlSndStarting	= new URL(cl.getResource("resources/sounds/login.wav").toString());
			URL urlSndLogin 		= new URL(cl.getResource("resources/sounds/login.wav").toString());
			URL urlSndLogout	= new URL(cl.getResource("resources/sounds/logout.wav").toString());
			URL urlSndMessage= new URL(cl.getResource("resources/sounds/message.wav").toString());
			URL urlSndChat 		= new URL(cl.getResource("resources/sounds/chat.wav").toString());
			URL urlSndFILE 		= new URL(cl.getResource("resources/sounds/file.wav").toString());
			*/
			//System.out.println(urlSndStarting.toString());
			/*
			URL urlLogin = new URL(
				//local.getProperty(MainFrame.SOUND_LOGIN_PROP, 
				cl.getResource("/resources/sounds/login.wav").toString()) );
			URL urlMessage1 = new URL(
				//local.getProperty(MainFrame.SOUND_MESSAGE_1_PROP, 
				cl.getResource("/resources/sounds/message1.wav").toString()) );
			URL urlMessage2 = new URL(
				//local.getProperty(MainFrame.SOUND_MESSAGE_2_PROP, 
				cl.getResource("/resources/sounds/message2.wav").toString()) );
*/
			waveMap.put(SOUND_STARTING, readBytes(urlSndStarting));
			waveMap.put( SOUND_LOGIN, readBytes(urlSndLogin) );
			waveMap.put( SOUND_LOGOUT, readBytes(urlSndLogout) );
			waveMap.put(SOUND_MESSAGE, readBytes(urlSndMessage));
			waveMap.put(SOUND_CHAT, readBytes(urlSndChat));
			waveMap.put(SOUND_FILE, readBytes(urlSndFILE));
			
		}
		catch( Exception e ) { e.printStackTrace(); }
	}

	private static byte[] readBytes( String path) throws IOException
	{
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		byte[] buf = new byte[ 8192 ];
		InputStream in = new FileInputStream(path);//path.openStream();
		try 
		{
			int readlen;
			while( (readlen=in.read(buf))>0 )
				bos.write( buf, 0, readlen );	
		}
		finally
		{
			in.close();
		}
		return bos.toByteArray();
	}

	
	public static void setEnabled(  boolean enable )
	{
		useSound = enable;
		//MsgUtil.println("snd  :" + enable + useSound);
	}
/*
	public static boolean isEnabled( String name )
	{
		return acceptMap.containsKey(name);
	}
*/
	/**
	 * 해당 이름에 해당하는 Wave 파일을 연주한다. 
	 * 이 method는 비동기로써, 곧바로 return된다.
	 */
	public static void play( String name )
	{
		//MsgUtil.println("snd  :" + useSound);
		if(!useSound)
			return ;
		if(musicBoxLastPlayedTime+1 >(System.currentTimeMillis()/1000)){
			return ;
			/*
			MusicBox.play(MusicBox.SOUND_STARTING);*/
			
		}
				
		musicBoxLastPlayedTime = (System.currentTimeMillis()/1000);
		
		
		byte[] dat = waveMap.get(name);
		if( dat==null )
			return;

		//if( !acceptMap.containsKey(name) )
//			return;

		try
		{
			AudioInputStream ais = AudioSystem.getAudioInputStream(
				new ByteArrayInputStream(dat) );

			AudioFormat format = ais.getFormat();
			DataLine.Info info = new DataLine.Info( Clip.class, ais.getFormat(),
				((int)ais.getFrameLength() * format.getFrameSize()) );
			Clip clip = (Clip)AudioSystem.getLine( info );
			clip.addLineListener( clipListener );
			clip.open( ais );
			clip.start();
		}
		catch (LineUnavailableException e) {
			MsgUtil.errPrintln("Audio Device Unavailable");
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}
	}
	public static void main(String[] args) throws InterruptedException{
		MusicBox.init();
		//MusicBox.play(MusicBox.SOUND_MESSAGE);
		MusicBox.play(SOUND_STARTING);
		Thread.sleep(3000);
		
	}

	/**
	 * 연주가 끝나면 해당 Clip을 close하기 위한 리스너이다.
	 * 구지 Thread를 생성하는 것은, Redhat 7.1, JDK 1.4에서 
	 * 곧바로 close() 할경우 VM Panic을 일으켰기 때문이다.
	 */
	private static class ClipListener implements LineListener 
	{
		public void update( final LineEvent e )
		{
			if( e.getType().equals(LineEvent.Type.STOP) )
			{
				new Thread( new Runnable() {
					public void run()
					{
						e.getLine().close();
					}
				}).start();
			}
		}
	}
}
