package kfmes.jateon.swt.dialog;


import java.io.IOException;

import kfmes.jateon.swt.Main;
import kfmes.jateon.swt.MainFrame;
import kfmes.jateon.swt.UIToolKit;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * 웹 브라우저를 구현한 클래스
 * @author kfmes(KIM Ga-Hyeon, jateon@kfmes.com, http://jateon.kfmes.com ) 
 * @version $Id: WebBrowser.java,v 1.15 2007/11/19 10:13:30 kfmes Exp $
 */
final public class WebBrowser {
	public static final String APP_TITLE = "JaTeOn Browser";

	public static final String HOME_URL = "http://jateon.kfmes.com";

	private org.eclipse.swt.widgets.Shell sShell = null;  //  @jve:decl-index=0:visual-constraint="10,-5"
	//public static boolean isAvailable = false;

	private Button backButton = null;
	private Button forwardButton = null;
	private Button stopButton = null;
	private Text locationText = null;
	private Button goButton = null;
	private Browser browser = null;
	private Button homeButton = null;
	private Label statusText = null;
	private ProgressBar progressBar = null;
	private Button refreshButton = null;
	
	public static final int MODE_NORMAL = 0;
	public static final int MODE_CYMINIHP = 1;
	private int mode = 0;

	private Shell sShell1 = null;  //  @jve:decl-index=0:visual-constraint="200,24"
	

		
	/**
	 * This method initializes browser	
	 *
	 */
	private void createBrowser() {
		org.eclipse.swt.layout.GridData gridData3 = new org.eclipse.swt.layout.GridData();
		browser = new Browser(sShell, SWT.BORDER);
		gridData3.horizontalSpan = 7;
		gridData3.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData3.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData3.grabExcessVerticalSpace = true;
		browser.setLayoutData(gridData3);
		browser.addTitleListener(new org.eclipse.swt.browser.TitleListener() {
			public void changed(org.eclipse.swt.browser.TitleEvent e) {
				sShell.setText(APP_TITLE + " - " + e.title);
			}
		});
		browser
				.addLocationListener(new org.eclipse.swt.browser.LocationListener() {
					public void changing(org.eclipse.swt.browser.LocationEvent e) {
						locationText.setText(e.location);
					}

					public void changed(org.eclipse.swt.browser.LocationEvent e) {
					}
				});
		browser
				.addProgressListener(new org.eclipse.swt.browser.ProgressListener() {
					public void changed(org.eclipse.swt.browser.ProgressEvent e) {
						if (!stopButton.isEnabled() && e.total != e.current) {
							stopButton.setEnabled(true);
						}
						progressBar.setMaximum(e.total);
						progressBar.setSelection(e.current);
					}

					public void completed(
							org.eclipse.swt.browser.ProgressEvent e) {
						stopButton.setEnabled(false);
						backButton.setEnabled(browser.isBackEnabled());
						forwardButton.setEnabled(browser.isForwardEnabled());
						progressBar.setSelection(0);
					}
				});
		browser
				.addStatusTextListener(new org.eclipse.swt.browser.StatusTextListener() {
					public void changed(
							org.eclipse.swt.browser.StatusTextEvent e) {
						statusText.setText(e.text);
					}
				});
		browser.setUrl(HOME_URL);
	}
	
	
	
	
	/**
	 * Dummy Constructor
	 *
	 */
	public WebBrowser() {	}
	
	public WebBrowser(boolean external , String URL){
		if(!external)
			return;
		String Browser = null;
		if(browser==null && Main.osname.startsWith("windows")){
			Browser = "C:\\Program Files\\Internet Explorer\\IEXPLORE.EXE";
			//Browser = "start IEXPLORE.EXE";
		}
		else if(browser == null )
			Browser = "firefox";
		
		Runtime rt = Runtime.getRuntime();
		
		try {
			
			rt.exec(Browser + " " + URL);
		} catch (IOException e) {
			MainFrame.showMessageBox(
					"브라우저 시작 실패", "브라우저를 시작하는데 실패했습니다", 
					SWT.ICON_ERROR);
		}
		
	}
	
	public WebBrowser(MainFrame main, String URL) {
		
		try{
			Display display = org.eclipse.swt.widgets.Display.getDefault();
			createSShell();
			sShell.setImage(UIToolKit.DefaultImage);
			browser.setUrl(URL);
			main.addWindow(sShell);
			sShell.setSize(800, 600);
			sShell.open();
			
			while (!sShell.isDisposed()) {
				if (!display.readAndDispatch())
					display.sleep();
			}
			
		}catch (Throwable e) {
			MainFrame.showMessageBox("에러", "웹 브라우저에서 에러가 발생했습니다" +  e.toString(), SWT.ICON_ERROR);
		}
		finally{
			main.removeWindow(sShell);
			sShell.dispose();
		}
	}
	public WebBrowser(MainFrame main , String URL, int mode) {
		try{
			this.mode = mode;
			Shell sShell = null;
			Display display = org.eclipse.swt.widgets.Display.getDefault();
			if(mode==MODE_NORMAL){
				createSShell();
				sShell = this.sShell;			
			}
			else if(mode == MODE_CYMINIHP){
				createSShell1();
				sShell = this.sShell1;
			}
			
			sShell.setImage(UIToolKit.DefaultImage);
			main.addWindow(sShell);
			sShell.open();
			
			browser.setUrl(URL);
			
			
			while (!sShell.isDisposed()) {
				if (!display.readAndDispatch())
					display.sleep();
			}
			main.removeWindow(sShell);
			sShell.dispose();
		}catch (Throwable e) {
			MainFrame.showMessageBox("에러", "웹 브라우저에서 에러가 발생했습니다" +  e.toString(), SWT.ICON_ERROR);
		}
		finally{
			main.removeWindow(sShell);
			if(sShell!=null)
				sShell.dispose();
		}
		

	}


	/**
	 * This method initializes sShell1	
	 *
	 */
	private void createSShell1() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 0;
		gridLayout.marginWidth = 0;
		gridLayout.marginHeight = 0;
		gridLayout.verticalSpacing = 0;
		GridData gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.grabExcessVerticalSpace = true;
		sShell1 = new Shell();
		sShell1.setLayout(gridLayout);
		sShell1.setSize(new Point(940	,595));
		browser = new Browser(sShell1, SWT.BORDER);
		browser.setLayoutData(gridData);
		
		
		
		browser.addTitleListener(new org.eclipse.swt.browser.TitleListener() {
			public void changed(org.eclipse.swt.browser.TitleEvent e) {
				sShell1.setText(APP_TITLE + " - " + e.title);
			}
		});
		
	}





	/**
	 * This method initializes sShell
	 */
	private void createSShell() {
		sShell = new org.eclipse.swt.widgets.Shell();
		org.eclipse.swt.layout.GridLayout gridLayout1 = new GridLayout();
		org.eclipse.swt.layout.GridData gridData2 = new org.eclipse.swt.layout.GridData();
		org.eclipse.swt.layout.GridData gridData4 = new org.eclipse.swt.layout.GridData();
		org.eclipse.swt.layout.GridData gridData5 = new org.eclipse.swt.layout.GridData();
		org.eclipse.swt.layout.GridData gridData6 = new org.eclipse.swt.layout.GridData();
		org.eclipse.swt.layout.GridData gridData7 = new org.eclipse.swt.layout.GridData();
		org.eclipse.swt.layout.GridData gridData8 = new org.eclipse.swt.layout.GridData();
		backButton = new Button(sShell, SWT.ARROW | SWT.LEFT);
		forwardButton = new Button(sShell, SWT.ARROW | SWT.RIGHT);
		stopButton = new Button(sShell, SWT.NONE);
		refreshButton = new Button(sShell, SWT.NONE);
		homeButton = new Button(sShell, SWT.NONE);
		locationText = new Text(sShell, SWT.BORDER);
		goButton = new Button(sShell, SWT.NONE);
		createBrowser();
		progressBar = new ProgressBar(sShell, SWT.BORDER);
		statusText = new Label(sShell, SWT.NONE);
		sShell.setText(APP_TITLE);
		sShell.setLayout(gridLayout1);
		gridLayout1.numColumns = 7;
		backButton.setEnabled(false);
		backButton.setToolTipText("이전 페이지");
		backButton.setLayoutData(gridData6);
		forwardButton.setEnabled(false);
		forwardButton.setToolTipText("앞 페이지");
		forwardButton.setLayoutData(gridData5);
		stopButton.setText("중지");
		stopButton.setEnabled(false);
		stopButton.setToolTipText("중지");
		goButton.setText("이동");
		goButton.setLayoutData(gridData8);
		goButton.setToolTipText("해당 페이지로 갑니다");
		gridData2.grabExcessHorizontalSpace = true;
		gridData2.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData2.verticalAlignment = org.eclipse.swt.layout.GridData.CENTER;
		locationText.setLayoutData(gridData2);
		locationText.setText(HOME_URL);
		locationText.setToolTipText("주소를 입력하세요");
		homeButton.setText("시작페이지");
		homeButton.setToolTipText("시작 페이지로 갑니다");
		statusText.setText("완료");
		statusText.setLayoutData(gridData7);
		gridData4.horizontalSpan = 5;
		progressBar.setLayoutData(gridData4);
		progressBar.setEnabled(false);
		progressBar.setSelection(0);
		gridData5.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData5.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData6.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData6.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData7.horizontalSpan = 1;
		gridData7.grabExcessHorizontalSpace = true;
		gridData7.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		gridData7.verticalAlignment = org.eclipse.swt.layout.GridData.CENTER;
		gridData8.horizontalAlignment = org.eclipse.swt.layout.GridData.END;
		gridData8.verticalAlignment = org.eclipse.swt.layout.GridData.CENTER;
		refreshButton.setText("새로고침");
		refreshButton.setToolTipText("페이지를 새로 읽어옵니다");
		sShell.setSize(new org.eclipse.swt.graphics.Point(553, 367));
		locationText
				.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
					public void mouseUp(org.eclipse.swt.events.MouseEvent e) {
						locationText.selectAll();
					}
				});
		locationText.addKeyListener(new org.eclipse.swt.events.KeyAdapter() {
			public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
				// Handle the press of the Enter key in the locationText.
				// This will browse to the entered text.
				if (e.character == SWT.LF || e.character == SWT.CR) {
					e.doit = false;
					browser.setUrl(locationText.getText());
				}
			}
		});
		refreshButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(
							org.eclipse.swt.events.SelectionEvent e) {
						browser.refresh();
					}
				});
		locationText
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(
							org.eclipse.swt.events.SelectionEvent e) {
						browser.setUrl(locationText.getText());
					}
				});
		stopButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(
							org.eclipse.swt.events.SelectionEvent e) {
						browser.stop();
					}
				});
		backButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(
							org.eclipse.swt.events.SelectionEvent e) {
						browser.back();
					}
				});
		forwardButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(
							org.eclipse.swt.events.SelectionEvent e) {
						browser.forward();
					}
				});
		homeButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(
							org.eclipse.swt.events.SelectionEvent e) {
						browser.setUrl(HOME_URL);
					}
				});
		goButton
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(
							org.eclipse.swt.events.SelectionEvent e) {
						browser.setUrl(locationText.getText());
					}
				});
	}
}
