package kfmes.natelib;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketException;

import kfmes.natelib.msg.Message;
import kfmes.natelib.util.MsgUtil;

/**
 * SS, NS 중 공통으로 가지고 있는 부분을 포함한 추상 클래스이다.
 * @author kfmes(KIM Ga-Hyeon, jateon@kfmes.com, http://jateon.kfmes.com )
 * @version $Id: AbstractProcessor.java,v 1.13 2007/11/19 10:13:29 kfmes Exp $
 */

public abstract class AbstractProcessor extends Thread{

	public NateonMessenger nate = null;
	protected String server =null;
		
	
	
	protected Socket socket = null;
	protected InputStream in = null;
	//protected InputStreamReader in = null;
	 
	protected OutputStream out = null;
	//protected OutputStreamWriter
	protected String line = null;
	protected  int TrID = 1;
	//protected CharsetDecoder charDecoder = new CharsetDecoder()
	
	protected void makeConnection(String IPaddr, int port) throws IOException{
		closeConnection();
		socket = new Socket(IPaddr, port);
		this.in = socket.getInputStream();
		//this.in = new InputStreamReader(socket.getInputStream(), Charset.forName("UTF-8"));
		this.out = socket.getOutputStream();
		
	}
	
	public boolean isConnected(){
		return (socket==null || socket.isClosed()) ? false: true;
	}
	
	protected void closeConnection() /*throws IOException*/{
		try{
			if( in != null )
				in.close();
			if( out != null )
				out.close();
			if( socket != null )
				socket.close();
		}catch (Exception e) {
			e.printStackTrace();
		}
	}
	protected void setTrID(int t) {
		this.TrID = t;		
	}

	/****
	 * 메세지 보내는 Methods
	 ********/
	
	
	/**
	 * raw수준의 메세지를 전송한다. 
	 * @param msg
	 * @throws IOException
	 */
	protected void send(String msg) throws IOException {
		if(socket.isConnected()){
			out.write(msg.getBytes("UTF8"));
			
			MsgUtil.println(">>>" + msg);
			TrID++;
		}
		//out.write(msg.toCharArray());
	}
	public void send(String header, String body) throws IOException{
		if(socket.isConnected()){
			String s = header + ' ' + TrID + ' ' +body + "\r\n"; 
			out.write(s.getBytes("UTF8"));
			
			MsgUtil.println(">>>" + s);
			TrID++;
			out.flush();
		}
	}
	/**
	 * Message 전송
	 * @param msg
	 * @throws IOException
	 */
	protected void send(Message msg) throws IOException {
		if(socket.isConnected()){
			out.write(msg.toString().getBytes("UTF8"));
			MsgUtil.println(">>>" + msg);
			TrID++;
		}
	}
	/**
	 * 소켓으로 메세지를 전송한다
	 * @param msgtype
	 * @param msg
	 * @throws IOException
	 */
	protected void sendMessage(String msgtype, String msg) throws IOException{
		if(socket.isConnected()){
		
			String ws;
			if(msg==null)
				ws = msgtype + " " + TrID + "\r\n";
			else
				ws = msgtype + " " + TrID + " " + msg +"\r\n";
		
			
			MsgUtil.print(">>>" + ws);
			out.write(ws.getBytes("UTF8"));
			TrID++;
		}
	}
	/**
	 * 소켓으로 메세지를 전송한다.
	 * sendMessage()와 다른점은 메세지 끝에 \r\n이 따라 붙지 않는다는 점이다.
	 * @param msgtype
	 * @param msg
	 * @throws IOException
	 */
	protected void sendMessageNoCrLf(String msgtype, String msg) throws IOException{
		if(socket.isConnected()){
			String ws;
			if(msg==null)
				ws = msgtype + " " + TrID ;
			else
				ws = msgtype + " " + TrID + " " + msg ;
			
			if(out==null)
				throw new SocketException();
			
			MsgUtil.print(">>>" + ws);
			out.write(ws.getBytes("UTF8"));
			TrID++;
		}
	}


	
	/*
	CTOC 534 ref@foo.com A 342
	IMSG
	title:음%20..네이트%20패킷%20연구중이어요%20-_-
	from:send@nate.com
	ref:ref@foo.com
	date:20060903005915
	20060903005915
	session_id:1509354
	uuid:1f3e9d5a-c071-4e71-bfa0-ef8cb27bf9da
	contenttype:text
	length:42
	font-name:굴림
	font-style:
	font-size:10
	font-color:#050505
		 */

	
	/**
	 * NS서버로부터 오는메세지 한줄을 읽는다.
	 * synchronized 되어있기때문에 사용시 주의를 요한다
	 * @return
	 * @throws IOException
	 */
	protected synchronized String readLine() throws IOException
	{
		ByteArrayOutputStream inbuf = new ByteArrayOutputStream();
		inbuf.reset();
		
		
		int v;
		
		while(true)
		{	
			try{
			
				if(socket==null || socket.isClosed())
					return null;

					if(in.available()>0){
						v = in.read();
					
						if( v==-1 )
							return null;
						if( v==13 )
							continue;
						if( v==10 )
							break;
						inbuf.write(v);
					}
					else{
						try{
							//yield();
							sleep(10);
						}
						catch (Exception e) {
							e.printStackTrace();
						}
					}
			
					
					
			}catch (SocketException e) {
				return null;
				// TODO: handle exception
			}
		}
		
		return new String(inbuf.toByteArray(),"UTF8");
	}
	/**
	 *  NS서버로부터 오는메세지를 size byte만큼 읽는다.
	 *  사용할때 size크기를 신중히 넣어야되고, 
	 *  이것또한 synchronized 되어있기때문에 사용시 주의를 요한다	
	 *  @param size
	 * @return
	 * @throws IOException
	 */
	protected synchronized String read(int size) throws IOException
	{
		ByteArrayOutputStream inbuf = new ByteArrayOutputStream();
		inbuf.reset();

		for(int i =0 ;i<size;i++){
			int v = in.read();
			inbuf.write(v);
		}		

		
		return new String(inbuf.toByteArray(),"UTF8");
	}

	protected synchronized void flushInputStream() throws IOException{
		long end = System.currentTimeMillis() + 1000;
		while(in.available()>0 || end<System.currentTimeMillis()){
			 int c = in.read();
			 //System.out.println();
			
		}
	}

	
	protected String recv() throws IOException{		
		String recv = null;
		while(true){
			recv = readLine();
						
			if(recv==null || recv.equals("null"))
				continue;
			else break;
		}
		
		MsgUtil.println("<<<" + recv);
		return recv;
	}
	
/*
	public int getTrid(){
		return TrID;
	}	
	*/
	/**
	 * @param args
	 */
	public static void main(String[] args) {


	}

}
