package kfmes.natelib;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import kfmes.natelib.entity.NateFile;
import kfmes.natelib.entity.NateFriend;
import kfmes.natelib.ftp.FileRecver;
import kfmes.natelib.ftp.FileSender;
import kfmes.natelib.msg.MimeMessage;
import kfmes.natelib.util.MsgUtil;


/**
 * 스위치보드 세션을 관리한다.
 * 
 * @author kfmes(KIM Ga-Hyeon, jateon@kfmes.com, http://jateon.kfmes.com )
 * @version $Id: SwitchBoardSession.java,v 1.49 2007/11/19 10:13:29 kfmes Exp $
 */

public class SwitchBoardSession extends AbstractProcessor {

	protected boolean isRunning = false;
	public static int timeout = 180000;
	private String sessionId = "";
	private NateFriend lastFriend = null;
	private NateFriend owner = null;
	
	private HashMap<String,NateFriend> friendMap = new HashMap<String,NateFriend>();
	private String sessionKey = null;
	
	private int port ;


	public SwitchBoardSession(NateonMessenger nate, String server, int port) {
		this(nate,server,port,null);
	}
	//
	public SwitchBoardSession(NateonMessenger nate,  String server, int port, String sessionKey) {
		this.nate = nate;
		//this.owner = owner;
		this.owner = nate.getOwner();
		this.server = server;
		this.port = port;
		this.sessionKey = sessionKey;
		init();
				
	}
	
//ENTR 1 id@nate.com [가현]%20자바를%20잡아잡아
//	김가현 630634BD36D96BD71DC9005B7F8B UTF8 P
//
	
	
	public void init(){
		try{
			super.makeConnection(this.server, this.port);
			super.setTrID(1);
			String foo =  owner.getID() + " " + owner.getNickName() 
			+ " " + owner.getRealName() + " " + this.sessionKey + " UTF8 P";
			//MsgUtil.println("$$$$" + foo.getBytes("UTF-8"));
			
			sendMessage("ENTR", foo);
			
			
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	
		this.isRunning = true;
		this.start();
	}
	
	/**
	 * 현재 세션 id를 문자열 형태로 반환해준다.
	 */
	public String getSessionId()
	{
		return this.sessionId;
	}

	/**
	 * 현재 세션의 id를 문자열 형태로 설정한다.
	 */
	public void setSessionId( String sessionId )
	{
		this.sessionId = sessionId;
	}

	public NateFriend getOwner() {
		return nate.getOwner();
	}

	/**
	 * 특정 사용자를 이 세션에 추가한다.
	 */
	
	//<<<JOIN 0 id@nate.com [가현]%20자바를%20잡아잡아 김가현
	//<<<1 1 id@nate.com [가현]%20자바를%20잡아잡아 김가현
	protected void addNateFriend( String line )
	{
		String list[] = line.split(" ");
		//if(ns)
		//NateFriend f = new NateFriend(list[2],list[3],list[4]);
		NateFriend f =nate.getNS().friendMap.get(list[2]);
		if(f==null)
			f = new NateFriend(list[2],list[2],"");
		
		friendMap.put( list[2],  f);
		this.lastFriend = f;
	}
	

	
	protected void addNateFriend( NateFriend friend )
	{
		friendMap.put( friend.getID(), friend );
		this.lastFriend = friend;
	}
	

	/**
	 * 가장 최근에 이 세션에 들어온 친구를 얻어온다.
	 */
	public NateFriend getNateFriend()
	{
		if(lastFriend==null)
			return null;
		return this.lastFriend;
	}

	public NateFriend getNateFriend(String loginName){
		return friendMap.get(loginName);
	}

	/**
	 * 주어진 loginName을 가진 사용자가 이 세션에 물려있는지 아닌지
	 * 확인한다.
	 */
	public boolean isInFriend( String loginName )
	{
		return friendMap.containsKey(loginName);
	}

	/**
	 * 특정 loginName을 가진 사용자를 이 세션에서 제거한다.
	 */
	protected NateFriend removeMsnFriend( String loginName )
	{
		return friendMap.remove( loginName );
	}

	public Collection getNateFriends()
	{
		return friendMap.values();
	}

	/**
	 * 현재 이 세션에 연결되어있는 친구의 수를 얻어온다.
	 * 자기 자신은 이 숫자에서 제외된다.
	 */
	public int getFriendCount()
	{
		return friendMap.size();
	}
	/*
	@Override
	protected void sendMessage(String msgtype, String message) throws IOException {
		// TODO 자동 생성된 메소드 스텁
		String msg = new String(message.getBytes());
		super.sendMessage(msgtype, msg);
	}
	*/
	
	public boolean sendMessage(String message){
		
		if(this.isRunning==false){
			return false;
		}
		//String msg = new String(message.getBytes(),"UTF-8");
		message = message.replaceAll("%", "%25");
		message = message.replaceAll(" ", "%20");
		message = message.replaceAll("\t", "%20");
		message = message.replaceAll("\n", "%0A");
		final String message_ = message;
		
						
		try { 
			if(getFriendCount()==0)
				throw new Exception("No Recver");
			super.sendMessage("MESG", "MSG " + message);
			
			new Runnable(){
				public void run() {
					int cur = (int)(System.currentTimeMillis()/1000);//+10;
					int limit = cur+10;
					int tid= TrID;
					String ar[] = null;
					String msg = message_;
					String tline = "";
					
					while(true){
						
						cur = (int)(System.currentTimeMillis()/1000);//+10;			
						if(cur>limit) 
							return;
						if(tline.equals(line)){
							tline = line;
							ar = tline.split(" ");
							if(ar[0].equals("280") && ar[1].equals(tid+"")) // send error
								procSendError(msg);							
						}
					}//end while
				}//end run
			};//end runnable
			/*
			line = super.readLine();
			if(line.startsWith("280")) // 무슨 일인지는 몰라도 전송에 실패했을경우 
				return false;
			*/
			
		} catch (Exception e) {
			//procSendError(message);
			//procMESG("MESG 0 " + lastFriend.getNateID()+ " MSG 대화전송실패!");
			e.printStackTrace();
			return false;
		}
	return true;
	}
	
	public boolean sendRawMessage(String message){
		
		if(this.isRunning==false){
			return false;
		}
		//String msg = new String(message.getBytes(),"UTF-8");
		//message = message.replace(" ", "%20");

		
		try { 
			
			super.send( message + "\r\n");
		} catch (IOException e) {
			
			e.printStackTrace();
			return false;
		}
	return true;
	}
	
	
	public void progressTyping(int flag ){
		if(this.isRunning==false)
			return ;
						
		try {			
			super.sendMessage("MESG", "TYPING " + flag);
		} catch (IOException e) {
			
			e.printStackTrace();			
		}

	}

	public void run() {
		this.setName(toString());
		nate.fireSwitchboardSessionStarted(this);
		
		while(in!=null && isRunning){
			try{
				
				//if(line==null || line.equals(""))
				line = readLine();//
				//new String(readLine().getBytes("MS949"));
				//line = "";
				if(line!=null){
					MsgUtil.println("<<s:" + line);					
							
					//MsgUtil.println( foo + "/" + bar);
					if(line.equals("null"))
						break;
					else if(line.startsWith("PING "))
						sendMessage("PING", null);
					else if(line.startsWith("USER")){
						//USER 0 1 1 id@nate.com [가현]%20자바를%20잡아잡아 김가현
						addNateFriend(line.substring(7));
					}
					else if(line.startsWith("JOIN 0"))
						addNateFriend(line);
					else if(line.startsWith("MESG 0"))
						procMESG(line);
					else if(line.startsWith("WHSP 0"))
						procWHSP(line);
					/*
					else if(line.startsWith("280"))
						procSendError();*/
					else if(line.startsWith("QUIT 0")){ // 대화방에서 나갔다는 신호
						String list[] = line.split(" ");
						MimeMessage mime = new MimeMessage(list[2],"QUIT");
						mime.setType("QUIT");
						
						nate.fireChatMessageReceived(this, getNateFriend(list[2]),mime);
						nate.fireSwitchboardSessionEnded(this);
						friendMap.remove(list[2]);
						if(friendMap.size()==0)
							this.close();
					}
					/*else if(line.startsWith("RESS"))
						MsgBuffer = line;*/
					
				}
					
				}catch (Exception e) {
					e.printStackTrace();
				}
				
			}
		
		closeConnection();
		nate.fireSwitchboardSessionEnded(this);
		MsgUtil.println("종료됨 : " + this.toString());
	}
	

	private void procWHSP(String line) {
		String ll[] = line.split(" ");
		NateFriend ref = friendMap.get(ll[2]);
		
		
		if(ll[3].equals("FILE") && ll[4].startsWith("REQUEST")){// 파일 수신 요청
			String tokens[] = line.split("%09");
			int filecnt = Integer.parseInt(tokens[1]);
			ArrayList<NateFile> list = new ArrayList<NateFile>();
			//MsgUtil.prints(tokens);
			for(int i=0;i<filecnt;i++){
				list.add(new NateFile(tokens[i+2]));
			}
			FileRecver recv = new FileRecver();
			recv.setRef(ref);
			recv.addFileList(list);
			nate.fireFilePosted(this, recv, ref, list);		
			
		}
		else if(ll[3].equals("FILE") && ll[4].startsWith("CANCEL")){
			String tokens[] = line.split("%09");
			nate.filemgr.stopRecv(tokens[2]);
			//MsgUtil.prints(tokens);
		}
		
	

		
	}
	public void sendFileReject(NateFriend ref, FileRecver recv){// String filename, int filelength , String tag) {
		ArrayList<NateFile> list = recv.getFileList();
		
		try{
			NateFile file = list.get(0);
			nate.filemgr.removeFileRecever(recv);// .removeRecvFile(tag);
			sendMessage("WHSP", ref.getNateID() + " FILE NACK%091%09" + file.fileName+ "|" + file.fileSize + "|"+file.getCookie());
			
			//sendMessage("WHSP","");
		}catch (Exception e) {
			e.printStackTrace();
		}
	}
	public void sendFileCancel(NateFriend ref, FileSender s) {
		ArrayList<NateFile> list = s.getFileList();
		
		try{
			NateFile file = list.get(0);
			nate.filemgr.removeFileSender(s);
			sendMessage("WHSP", ref.getNateID() + " FILE NACK%091%09" + file.fileName+ "|" + file.fileSize + "|"+file.getCookie());

		}catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	private void procMESG(String line) {
		/*
<<<MESG 0 id@nate.com TYPING 2
<<<MESG 0 id@nate.com TYPING 1
		 * 0 - MESG
		 * 1 - TrID
		 * 2 - loginName
		 * 3 - msg type (MSG | TYPING 0,1,2);
		 * 4 - msg ( MESSAGE);
		 */
		String list[] = line.split(" ");
		
		if(list[3].equals("MSG")){// 상대방으로부터 대화가 도착했을때
				
				MimeMessage mime = new MimeMessage(line);
				NateFriend f = friendMap.get(list[2]);
				if(f==null)
					f = new NateFriend(list[2]);
				
				nate.fireChatMessageReceived(this, f, mime);
				
		}
		else if(list[3].equals("TYPING")){
			if(list[4].equals("0"))
				nate.fireProgressTyping(this, (NateFriend) friendMap.get(list[2]), 0);
			else if(list[4].equals("1"))
				nate.fireProgressTyping(this, (NateFriend) friendMap.get(list[2]), 1);
				
		}
		
		
	}
	private void procSendError(String msg) {		
		String lastid = (lastFriend==null) ? "null" : this.lastFriend.getNameID();
		MimeMessage mime = new MimeMessage(lastid, false );
		mime.setMessage(msg);
		nate.fireChatMessageReceived(this, this.lastFriend, mime);
		
	}
	
	
	public void close(){
		
		isRunning = false;
		
		if(super.isConnected()){
			try{
				send("QUIT 0\r\n");
				//interrupt();
				super.closeConnection();
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		//sendMessage("QUIT");
		nate.sessionList.remove(this);
		
		nate.fireSwitchboardSessionEnded(this);

		
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {

	}


}
