package kfmes.natelib.event;

import java.util.ArrayList;
import java.util.EventListener;
import java.util.Properties;

import kfmes.natelib.SwitchBoardSession;
import kfmes.natelib.entity.GroupList;
import kfmes.natelib.entity.NateFile;
import kfmes.natelib.entity.NateFriend;
import kfmes.natelib.ftp.FileRecver;
import kfmes.natelib.ftp.FileSender;
import kfmes.natelib.msg.InstanceMessage;
import kfmes.natelib.msg.MimeMessage;


/**
 *  네이트온 이벤트 리스너
 * @author kfmes(KIM Ga-Hyeon, jateon@kfmes.com, http://jateon.kfmes.com )
 * @version $Id: NateListener.java,v 1.22 2007/11/19 10:13:37 kfmes Exp $
 */
public interface NateListener extends EventListener{
	
	/**
	 * 쪽지가 왔을때 발생한다
	 * @param imsg
	 */
	public void instanceMessageReceived(InstanceMessage imsg);
	
	/** ---------------------------------------------**/
	/**
	 * 로그인이 완료되었다는 메시지이다.
	 * 자신의 로그인이름(메일주소)와 닉네임을 던져준다.
	 */
	public void loginComplete( );

	/**
	 * 존재하지 않는 아이디이거나, 비밀번호가 틀렸을 경우,
	 * 이 이벤트를 발송하고, 연결은 종료된다.
	 * 아무튼 로그인을 할 수 없을때 발생되는 이벤트이다.
	 */
	public void loginError( String header );

	/**
	 * LST 메시지 도착시 알려준다.
	 */
	public void listAdd( NateFriend friend );

	/**
	 * 로그인 한 후, 상태를 온라인으로 바꾸었을때, 자신의
	 * Contact list에 있는 사용자중에 상태가 Online(혹은 substate)인
	 * 사용자들을 이 메소드를 통해 임의의 길이로 날려준다.
	 * 문제점은 호출 종료지점을 정확하게 알 수 없다는 것이다.
	 * <p>
	 * 만약 Online Contact list를 가지고 싶다면, MsnFriend 객체를
	 * Map에 저장해두면 편리하다. (Key값은 loginName으로 하면 더 좋다)
	 */
	public void listOnline( NateFriend friend );

	/**
	 * 자신의 ContactList에 있는 사용자 중에 한명이 온라인이 되거나
	 * 상태를 변경하였을때, NS로부터 날라오는 메시지이다.
	 */
	public void userOnline( NateFriend friend );

	/**
	 * 자신의 Foward ContactList에 있는 사용자중 온라인이였던 사용자가
	 * 오프라인으로 상태가 변경되었을때 NS로부터 날라오는 메시지이다.
	 */
	public void userOffline( NateFriend friend );


	/**
	 * 누군가가 Switchboard server를 통해서 자신에게 대화요청을
	 * 하였고, 그 사람과의 session이 연결되었다는 것을 알려준다.
	 */
	public void switchboardSessionStarted( SwitchBoardSession ss );

	/**
	 * 특정 세션으로 새로운 사용자가 입장하였음을 알려준다.
	 */
	public void whoJoinSession( SwitchBoardSession ss, NateFriend join );

	/**
	 * 특정 세션에서 사용자가 BYE 하고 세션을 끊었음을 알려준다.
	 */
	public void whoPartSession( SwitchBoardSession ss, NateFriend part );

	/**
	 * Switchboard session이 상대방으로부터 종료되었을때 불려진다.
	 */
	public void switchboardSessionEnded( SwitchBoardSession ss );

	/**
	 * Switchboard session이 연결을 완료하지 못하고 SS로부터 연결이 끝어져버렸을때
	 * 발생하는 이벤트이다. SwitchboardSession의 Session id는 null일 것이다.
	 */
	public void switchboardSessionAbandon( SwitchBoardSession ss, String targetName );

	/**
	 * 특정 Switchboard session에서 typingUser사용자가 메시지를 입력하는
	 * 중일때 발송될때 불려진다.
	 */
	public void progressTyping( SwitchBoardSession ss,
			NateFriend friend , int flag);

	/**
	 * 특정 Switchboard session에서 인스턴트 메시지를 받았을때 불려진다.
	 *
	 * @param  ss   해당 switchboard session
	 * @param  msg
	 */
	public void chatMessageReceived( SwitchBoardSession ss,
			NateFriend friend, MimeMessage mime );

	/**
	 * 누군가가 자신에게 파일을 보내려고 함을 알려주는 메소드이다.
	 *
	 */
	//public void filePosted( SwitchBoardSession ss,String ref, String cookie, String filename, int filesize );
	
	public void filePosted(SwitchBoardSession session, FileRecver recv, NateFriend ref, ArrayList<NateFile> list);

	public void fileSend(SwitchBoardSession session, FileSender sender, NateFriend ref, ArrayList<NateFile> list);


	/**
	 * 보내려고 했던 파일에 대하여 상대방이 Accept해주었음을 알려주는 메소드이다.
	 * 이 이벤트가 발생하였을 경우에 특별히 해줘야할일은 없다.
	 * 단 이 이벤트가 발생한 후, 곧바로 실제로 파일을 전송하기위한 thread가
	 * 생성될 것이며 port가 bind되고 파일 전송이 시작될 것이다.
	 */
	public void fileSendAccepted( SwitchBoardSession ss, String cookie );

	/**
	 * 보내려고 했던 파일에 대하여 상대방이 Reject했음을 알려주는 메소드이다.
	 * 이 이벤트가 발생하였을 경우에 특별히 해줘야할일은 없다.
	 */
	public void fileSendRejected( SwitchBoardSession ss, String cookie, String reason );

	/**
	 * 파일 보내기 작업이 시작되었음을 알려준다.
	 *
	 * @param  server  파일 송신 서버 객체.
	 */
	public void fileSendStarted( FileSender sender);

	/**
	 * 파일 보내기 작업이 종료되었음을 알려준다.
	 * @param natefile 
	 */
	public void fileSendEnded( FileSender sender, NateFile natefile);

	/**
	 * 파일 다운로드 작업이 시작되었음을 알려준다.
	 *
	 * @param  downloader  파일 다운로드 thread 객체.
	 */
	//public void fileReceiveStarted( VolatileDownloader downloader );

	/**
	 * 파일을 송신하던중 예외가 발생하였음을 알려주는 메소드이다.
	 * 어쨌든, 파일 전송 thread는 종료된다.
	 */
	public void fileSendError(FileSender sender, Throwable e );

	/**
	 * 파일을 수신하던중 예외가 발생하였음을 알려주는 메소드이다.
	 * 어쨌든, 파일 전송 thread는 종료된다.
	 */
	//public void fileReceiveError( VolatileDownloader downloader, Throwable e );

	/**
	 * 누군가 자신을 등록했음을 알려준다. RL만이 등록되어있으므로,
	 * 실제 자신의 FL에 등록하는 것은 자신이 직접해주어야 한다.
	 *
	 * @param  friend  등록한 사람의 LoginName, FriendlyName이 저장.
	 */
	public void whoAddedMe( NateFriend friend );

	/**
	 * 누군가 자신을 삭제했음을 알려준다. RL에서만 삭제되므로, 구지 알려줄 필요는 없다.
	 *
	 * @param  friend  등록한 사람의 LoginName만이 온다. FriendlyName은 null일 것이다.
	 */
	public void whoRemovedMe( NateFriend friend );

	/**
	 * BuddyList가 수정되어 Serial number가 변경된 직 후, 발송되는 이벤트이다.
	 */
	public void buddyListModified();

	/**
	 * 사용자 등록을 실패하였을때 발송되는 이벤트이다.
	 *
	 * @param  errorCode  서버로부터 받은 에러 코드 헤더
	 */
	public void addFailed( int errorCode );

	/**
	 * 친숙한 이름(FriendlyName)이 변경되었을때 발송되는 이벤트이다.
	 * 자신이 변경을 했을때도, 이 이벤트가 발송되며, 다른 인간이 변경해도
	 * 발송된다.
	 * <p>
	 * 만약 null이라면, 자기 자신의 이름 변경에 실패한 case이다.
	 */
	public void renameNotify( NateFriend friend );

	/**
	 * 로그인 시, Synchronization value가 달랐을때, 서버로부터
	 * FL/AL/BL/RL, Group list등을 모두 받게 되는데, 만만치 않은
	 * 작업이므로, 모두 다 Update되었을때 통지되는 이벤트이다.
	 */
	public void allListUpdated();

	/**
	 * 로그아웃 되었음을 알려주는 이벤트이다. 반드시 정상적인 로그아웃만을
	 * 알려주는 것은 아니다. Emergency exception을 통해 NotificationProcessor가
	 * 종료되었을 경우에도 이벤트가 발생한다.
	 * <p>
	 * 즉, 결국 connection이 terminate되었음을 알려주는 것이다.
	 */
	public void logoutNotify();

	/**
	 * HotMail 계정사용시, 로그인시 읽지않은 편지에 대한 정보를 넘겨준다.
	 * Properties는 MSN 서버에서 넘겨준 MIME properties들이며, 이 중
	 * Inbox-Unread property를 사용하면 된다. 이것은 편의상 unread variable로
	 * 전달된다.
	 */
	public void notifyUnreadMail( Properties prop, int unread );

	public void buddyListInit(GroupList grouplist);
	
	/**
	 * buddy의 상태가 바뀌었을때 사용한다
	 * @param friend
	 */
	public void buddyModified(NateFriend friend);
	
	/**
	 * kill msg
	 *
	 */
	public void killed();
	public void OwnerStatusUpdated();

	/**
	 * 파일 받기가 끝났을때 발생하는 이벤트
	 * @param recver
	 * @param natefile
	 */
	public void fileRecved(FileRecver recver, NateFile natefile);

	public void fileTransferAborted(String cookie);

	


}
